#The purpose of this script is to create a local wellhead system model in Abaqus, 
#based on wellhead/conductorhousing coordinates and p-y springs from file.
#The model is built as described in DNV "Wellhead Fatigue Analysis Method"
#Report no/DNV Reg No.: 2011-0063/ 12Q5071-26, Rev 01, 2011-01-18

#Written by John Marius Hegseth, M.Sc. thesis spring 2014.

from abaqus import *
from abaqusConstants import *
from caeModules import *
backwardCompatibility.setValues(includeDeprecated=True, reportDeprecated=False)

import sketch
import part

f = open('wellhead_coord.txt', 'r')
g = open('cond_house_coord.txt', 'r')

xzCoordinatesWell = []
xzCoordinatesCond = []
contentWell = f.readlines()
contentCond = g.readlines()

for line in contentWell:
    value = line.split()
    xzCoordinatesWell.append(value)

for line in contentCond:
    value = line.split()
    xzCoordinatesCond.append(value)

f.close
g.close

myModel = mdb.Model(name='Model-1')

#----------------------------Sketches-----------------------------------------#

#----------------------------Wellhead-----------------------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-1', sheetSize=10.0)
mySketch.ConstructionLine(point1=(0.0, -5.0), point2=(0.0, 5.0))

for i in range(len(xzCoordinatesWell)-1):
    mySketch.Line(point1=(float(xzCoordinatesWell[i][0]), float(xzCoordinatesWell[i][1])), 
        point2=(float(xzCoordinatesWell[i+1][0]), float(xzCoordinatesWell[i+1][1])))

myPart = myModel.Part(name='Wellhead', dimensionality=THREE_D, type=DEFORMABLE_BODY)
myPart.BaseSolidRevolve(sketch=mySketch, angle=180)

#-----------------------------Conductor Housing------------------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-2', sheetSize=10.0)
mySketch.ConstructionLine(point1=(0.0, -5.0), point2=(0.0, 5.0))

for i in range(len(xzCoordinatesCond)-1):
    mySketch.Line(point1=(float(xzCoordinatesCond[i][0]), float(xzCoordinatesCond[i][1])), 
        point2=(float(xzCoordinatesCond[i+1][0]), float(xzCoordinatesCond[i+1][1])))

mySketch.FilletByRadius(radius=0.0127, curve1=mySketch.geometry.findAt((0.385,-1.5551),), 
    nearPoint1=(0.385, -1.5551), curve2=mySketch.geometry.findAt((0.381,-1.56),), nearPoint2=(0.381, -1.56))

myPart = myModel.Part(name='ConductorHousing', dimensionality=THREE_D, type=DEFORMABLE_BODY)
myPart.BaseSolidRevolve(sketch=mySketch, angle=180)

#--------------------------------Conductor------------------------------------------#
Transition = 12 #Solid to beam transition 12 m below mud line
mySketch = myModel.ConstrainedSketch(name='Sketch-3', sheetSize=100.0)
mySketch.ConstructionLine(point1=(0.0, -50.0), point2=(0.0, 50.0))

xzCoordinates = ((0.381, -1.6313), (0.381, -4-Transition), (0.3556, -4-Transition), (0.3556, -1.6313), (0.381, -1.6313))

for i in range(len(xzCoordinates)-1):
    mySketch.Line(point1=xzCoordinates[i], point2=xzCoordinates[i+1])

myPart = myModel.Part(name='Conductor', dimensionality=THREE_D, type=DEFORMABLE_BODY)
myPart.BaseSolidRevolve(sketch=mySketch, angle=180)

#-----------------------------Surface Casing-----------------------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-4', sheetSize=100.0)
mySketch.ConstructionLine(point1=(0.0, -50.0), point2=(0.0, 50.0))

xzCoordinates = ((0.2682, -1.7126), (0.2682, -4-Transition), (0.235, -4-Transition), (0.235, -1.7126), (0.2682, -1.7126))

for i in range(len(xzCoordinates)-1):
    mySketch.Line(point1=xzCoordinates[i], point2=xzCoordinates[i+1])

myPart = myModel.Part(name='SurfaceCasing', dimensionality=THREE_D, type=DEFORMABLE_BODY)
myPart.BaseSolidRevolve(sketch=mySketch, angle=180)

#----------------------------Cement-----------------------------------#
Shortfall = 10 #Cement shortfall from mudline
mySketch = myModel.ConstrainedSketch(name='Sketch-5', sheetSize=100.0)
mySketch.ConstructionLine(point1=(0.0, -50.0), point2=(0.0, 50.0))

xzCoordinates = ((0.3556, -4-Shortfall), (0.3556, -4-Transition), (0.2682, -4-Transition), (0.2682, -4-Shortfall), (0.3556, -4-Shortfall))

for i in range(len(xzCoordinates)-1):
    mySketch.Line(point1=xzCoordinates[i], point2=xzCoordinates[i+1])

myPart = myModel.Part(name='Cement', dimensionality=THREE_D, type=DEFORMABLE_BODY)
myPart.BaseSolidRevolve(sketch=mySketch, angle=180)

#------------------------------BOP---------------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-6', sheetSize=100.0)
mySketch.Line(point1=(0.0, 0.0), point2=(0.0, 10.651))
myPart = myModel.Part(name='BOP', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
myPart.BaseShellExtrude(sketch=mySketch, depth=0.32)

#------------------------Beam Conductor----------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-7', sheetSize=100.0)
mySketch.Line(point1=(0.0, -4-Transition), point2=(0.0, -74.0))
myPart = myModel.Part(name='BeamConductor', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
myPart.BaseWire(sketch=mySketch)

#-----------------------Beam Surface Casing------------------------#
mySketch = myModel.ConstrainedSketch(name='Sketch-8', sheetSize=100.0)
mySketch.Line(point1=(0.0, -4-Transition), point2=(0.0, -74.0))
myPart = myModel.Part(name='BeamSurfaceCasing', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
myPart.BaseWire(sketch=mySketch)

#--------------------------Assembly--------------------------------#

myAssembly = myModel.rootAssembly
myAssembly.DatumCsysByDefault(CARTESIAN)
myPart = myModel.parts['Cement']
myAssembly.Instance(name='Cement-1', part=myPart, dependent=OFF)
myPart = myModel.parts['Conductor']
myAssembly.Instance(name='Conductor-1', part=myPart, dependent=OFF)
myPart = myModel.parts['ConductorHousing']
myAssembly.Instance(name='ConductorHousing-1', part=myPart, dependent=OFF)
myPart = myModel.parts['SurfaceCasing']
myAssembly.Instance(name='SurfaceCasing-1', part=myPart, dependent=OFF)
myPart = myModel.parts['Wellhead']
myAssembly.Instance(name='Wellhead-1', part=myPart, dependent=OFF)
myPart = myModel.parts['BOP']
myAssembly.Instance(name='BOP-1', part=myPart, dependent=OFF)
myPart = myModel.parts['BeamConductor']
myAssembly.Instance(name='BeamConductor-1', part=myPart, dependent=OFF)
myPart = myModel.parts['BeamSurfaceCasing']
myAssembly.Instance(name='BeamSurfaceCasing-1', part=myPart, dependent=OFF)

#-----------------Coupling between BOP and Wellhead------------------------------#

side1Edges1 = myAssembly.instances['BOP-1'].edges.findAt(((0.0, 0.0, 0.25), ))
region1=regionToolset.Region(side1Edges=side1Edges1)
side1Faces1 = myAssembly.instances['Wellhead-1'].faces.findAt(((-0.280311, 0.0, 0.027503), ))
region2=regionToolset.Region(side1Faces=side1Faces1)
myModel.ShellSolidCoupling(name='BOPWellhead', 
    shellEdge=region1, solidFace=region2, positionToleranceMethod=COMPUTED)

#-----------------Coupling conductor transition------------------------------#

verts1 = myAssembly.instances['BeamConductor-1'].vertices.findAt(((0.0, -4-Transition, 0.0), ))
region1=regionToolset.Region(vertices=verts1)
side1Faces1 = myAssembly.instances['Conductor-1'].faces.findAt(((-0.355044, -4-Transition, 0.045361), ))
region2=regionToolset.Region(side1Faces=side1Faces1)
myModel.Coupling(name='ConductorTransition', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=OFF, ur2=OFF, ur3=OFF)

#-----------------Coupling surface casing transition------------------------------#

verts1 = myAssembly.instances['BeamSurfaceCasing-1'].vertices.findAt(((0.0, -4-Transition, 0.0), ))
region1=regionToolset.Region(vertices=verts1)
side1Faces1 = myAssembly.instances['SurfaceCasing-1'].faces.findAt(((-0.240104, -4-Transition, 0.029977), ))
region2=regionToolset.Region(side1Faces=side1Faces1)
myModel.Coupling(name='SurfaceCasingTransition', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=OFF, ur2=OFF, ur3=OFF)

#--------------------------Material assignments--------------------------------#

myModel.Material(name='Steel')
myModel.materials['Steel'].Density(table=((7850.0, ), ))
myModel.materials['Steel'].Elastic(table=((210000000000.0, 0.3),))

myModel.Material(name='BeamSteel')
myModel.materials['BeamSteel'].Density(table=((7850.0, ), ))
myModel.materials['BeamSteel'].Elastic(table=((105000000000.0, 0.3),))

myModel.Material(name='Cement')
myModel.materials['Cement'].Density(table=((1560.0, ), ))
myModel.materials['Cement'].Elastic(table=((3500000000.0, 0.3),))

myModel.PipeProfile(name='Conductor', r=0.381, t=0.0508, 
    formulation=THICK_WALL)
myModel.PipeProfile(name='SurfaceCasing', r=0.2682, t=0.06655, 
    formulation=THICK_WALL)
myModel.RectangularProfile(name='BOP', a=3.8, b=4.5)

myModel.HomogeneousSolidSection(name='SolidSteel', 
    material='Steel', thickness=None)
myModel.HomogeneousSolidSection(name='SolidCement', 
    material='Cement', thickness=None)
myModel.BeamSection(name='BeamConductor', 
    integration=DURING_ANALYSIS, poissonRatio=0.3, profile='Conductor', 
    material='BeamSteel', temperatureVar=LINEAR, consistentMassMatrix=False)
myModel.BeamSection(name='BeamSurfaceCasing', 
    integration=DURING_ANALYSIS, poissonRatio=0.3, profile='SurfaceCasing', 
    material='BeamSteel', temperatureVar=LINEAR, consistentMassMatrix=False)
myModel.HomogeneousShellSection(name='BOP', 
    preIntegrate=OFF, material='Steel', thicknessType=UNIFORM, thickness=1.0, 
    thicknessField='', idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=5)

myPart = myModel.parts['Wellhead']
cells = myPart.cells.findAt(((-0.330289, -0.301535, 0.030911), ))
region = regionToolset.Region(cells=cells)
myPart.SectionAssignment(region=region, sectionName='SolidSteel', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['ConductorHousing']
cells = myPart.cells.findAt(((-0.446826, -1.056012, 0.031547), ))
region = regionToolset.Region(cells=cells)
myPart.SectionAssignment(region=region, sectionName='SolidSteel', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['Conductor']
cells = myPart.cells.findAt(((-0.352593, -10.543767, 0.046148), ))
region = regionToolset.Region(cells=cells)
myPart.SectionAssignment(region=region, sectionName='SolidSteel', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['SurfaceCasing']
cells = myPart.cells.findAt(((-0.233013, -10.570867, 0.030497), ))
region = regionToolset.Region(cells=cells)
myPart.SectionAssignment(region=region, sectionName='SolidSteel', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['Cement']
cells = myPart.cells.findAt(((-0.265932, -14.333333, 0.034806), ))
region = regionToolset.Region(cells=cells)
myPart.SectionAssignment(region=region, sectionName='SolidCement', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['BeamConductor']
edges = myPart.edges.findAt(((0.0, -29.75, 0.0), ))
region = regionToolset.Region(edges=edges)
myPart.SectionAssignment(region=region, sectionName='BeamConductor', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['BeamSurfaceCasing']
edges = myPart.edges.findAt(((0.0, -29.75, 0.0), ))
region = regionToolset.Region(edges=edges)
myPart.SectionAssignment(region=region, sectionName='BeamSurfaceCasing', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

myPart = myModel.parts['BOP']
#faces = myPart.faces.findAt(((-0.125, 3.333333, 0.0), ))
faces = myPart.faces.findAt(((0.0, 3.333333, 0.125), ))
region = regionToolset.Region(faces=faces)
myPart.SectionAssignment(region=region, sectionName='BOP', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)

#-----------------Assigning beam orientation.........................#

myPart = myModel.parts['BeamConductor']
edges = myPart.edges.findAt(((0.0, -29.75, 0.0), ))
region=regionToolset.Region(edges=edges)
myPart.assignBeamSectionOrientation(region=region, method=N1_COSINES, n1=(0.0, 0.0, -1.0))

myPart = myModel.parts['BeamSurfaceCasing']
edges = myPart.edges.findAt(((0.0, -29.75, 0.0), ))
region=regionToolset.Region(edges=edges)
myPart.assignBeamSectionOrientation(region=region, method=N1_COSINES, n1=(0.0, 0.0, -1.0))

myPart = myModel.parts['BOP']
faces = myPart.faces.findAt(((0.0, 3.333333, 0.125), ))
region = regionToolset.Region(faces=faces)
myPart.MaterialOrientation(region=region, 
    orientationType=GLOBAL, axis=AXIS_1, additionalRotationType=ROTATION_NONE, 
    localCsys=None, fieldName='')

#-------------------------Load Steps---------------------------------#

myModel.StaticStep(name='Preload', previous='Initial', 
    description='BOP and LMRP weight, riser tension ,and casing down weight', nlgeom=ON)
myModel.StaticStep(name='ResponseLoad', previous='Preload', 
    description='Shear force and bending moment from riser', nlgeom=ON)
myModel.steps['ResponseLoad'].setValues(
    timeIncrementationMethod=AUTOMATIC, maxInc=0.05, initialInc=0.05, noStop=OFF)

#-------------------------Contact-----------------------------------#

#-------------------Friction steel-steel-----------------------------#
myModel.ContactProperty('SteelSteel')
myModel.interactionProperties['SteelSteel'].TangentialBehavior(
    formulation=PENALTY, directionality=ISOTROPIC, slipRateDependency=OFF, 
    pressureDependency=OFF, temperatureDependency=OFF, dependencies=0, table=((
    0.15, ), ), shearStressLimit=None, maximumElasticSlip=FRACTION, 
    fraction=0.005, elasticSlipStiffness=None)
myModel.interactionProperties['SteelSteel'].NormalBehavior(
    pressureOverclosure=HARD, allowSeparation=ON, 
    constraintEnforcementMethod=DEFAULT)

#-------------------Friction steel-cement----------------------------#
myModel.ContactProperty('SteelCement')
myModel.interactionProperties['SteelCement'].TangentialBehavior(formulation=FRICTIONLESS)
myModel.interactionProperties['SteelCement'].NormalBehavior(
    pressureOverclosure=HARD, allowSeparation=ON, 
    constraintEnforcementMethod=DEFAULT)

#----------Friction steel-steel during preload (frictionless)------------#
myModel.ContactProperty('SteelSteelPreload')
myModel.interactionProperties['SteelSteelPreload'].TangentialBehavior(formulation=FRICTIONLESS)
myModel.interactionProperties['SteelSteelPreload'].NormalBehavior(
    pressureOverclosure=HARD, allowSeparation=ON, 
    constraintEnforcementMethod=DEFAULT)

#------------------Surface casing - cement (fixed)-----------------------#
side1Faces1 = myAssembly.instances['SurfaceCasing-1'].faces.findAt(((-0.265932, -6.141733, 0.034806), ))
region1=myAssembly.Surface(side1Faces=side1Faces1, name='SurfaceCasingOuter')
side1Faces1 = myAssembly.instances['Cement-1'].faces.findAt(((-0.265932, -14.333333, 0.034806), ))
region2=myAssembly.Surface(side1Faces=side1Faces1, name='CementInner')
myModel.Tie(name='SurfaceCasingCement', master=region1, slave=region2, 
    positionToleranceMethod=COMPUTED, adjust=ON, tieRotations=ON, thickness=ON)

#------------------Conductor - cement (frictionless)----------------------#
side1Faces1 = myAssembly.instances['Conductor-1'].faces.findAt(((-0.352593, -10.543767, 0.046148), ))
region1=myAssembly.Surface(side1Faces=side1Faces1, name='ConductorInner')
side1Faces1 = myAssembly.instances['Cement-1'].faces.findAt(((-0.352593, -14.666667, 0.046148), ))
region2=myAssembly.Surface(side1Faces=side1Faces1, name='CementOuter')
myModel.SurfaceToSurfaceContactStd(name='ConductorCement', 
    createStepName='Initial', master=region1, slave=region2, sliding=FINITE, 
    thickness=ON, interactionProperty='SteelCement', adjustMethod=NONE, 
    initialClearance=OMIT, datumAxis=None, clearanceRegion=None)

#-------------------Conductor housing - conductor (weld)------------------#
side1Faces1 = myAssembly.instances['ConductorHousing-1'].faces.findAt(((-0.361095, -1.6313, 0.026376), ))
region1=regionToolset.Region(side1Faces=side1Faces1)
side1Faces1 = myAssembly.instances['Conductor-1'].faces.findAt(((-0.355044, -1.6313, 0.045361), ))
region2=regionToolset.Region(side1Faces=side1Faces1)
myModel.Tie(name='ConductorHousingConductor', master=region2, slave=region1, 
    positionToleranceMethod=COMPUTED, adjust=ON, tieRotations=ON, thickness=ON)

#----------------------Wellhead - surface casing--------------------------#
side1Faces1 = myAssembly.instances['Wellhead-1'].faces.findAt(((-0.241343, -1.7126, 0.026792), ))
region1=regionToolset.Region(side1Faces=side1Faces1)
side1Faces1 = myAssembly.instances['SurfaceCasing-1'].faces.findAt(((-0.240104, -1.7126, 0.029977), ))
region2=regionToolset.Region(side1Faces=side1Faces1)
myModel.Tie(name='WellheadSurfaceCasing', master=region2, slave=region1, 
    positionToleranceMethod=COMPUTED, adjust=ON, tieRotations=ON, thickness=ON)

#--------------------Wellhead - conductor housing-------------------------#

#--------------------------During Preload---------------------------------#
side1Faces1 = myAssembly.instances['ConductorHousing-1'].faces.findAt(((-0.392692, -0.925037, 0.029373), ), ((-0.378217, 
    -1.02151, 0.027763), ), ((-0.354609, -1.438327, 0.026524), ))
region1=myAssembly.Surface(side1Faces=side1Faces1, name='ContactSurfaceCH')

side1Faces1 = myAssembly.instances['Wellhead-1'].faces.findAt(((-0.391911, -0.917673, 0.0342), ), ((-0.376569, 
    -1.022958, 0.034137), ), ((-0.353104, -1.382987, 0.033614), ))
region2=myAssembly.Surface(side1Faces=side1Faces1, name='ContactSurfaceW')

myModel.SurfaceToSurfaceContactStd(name='ContactPreload', 
    createStepName='Preload', master=region1, slave=region2, sliding=FINITE, 
    thickness=ON, interactionProperty='SteelSteelPreload', adjustMethod=NONE, 
    initialClearance=OMIT, datumAxis=None, clearanceRegion=None)
myModel.interactions['ContactPreload'].deactivate('ResponseLoad')

#-----------------------During response load-----------------------------#
myModel.SurfaceToSurfaceContactStd(name='ContactResponseLoad', 
    createStepName='ResponseLoad', master=region2, slave=region1, sliding=FINITE, 
    thickness=ON, interactionProperty='SteelSteel', adjustMethod=NONE, 
    initialClearance=OMIT, datumAxis=None, clearanceRegion=None)

#--------------------------------Meshing----------------------------------#

#----Creating partition for upper part of surface casing and conductor----#
myAssembly.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=-2.0)
c1 = myAssembly.instances['SurfaceCasing-1'].cells
pickedCells = c1.findAt(((-0.233013, -10.570867, 0.030497), ))
d1 = myAssembly.datums
myAssembly.PartitionCellByDatumPlane(datumPlane=d1[34], cells=pickedCells)
c1 = myAssembly.instances['Conductor-1'].cells
pickedCells = c1.findAt(((-0.352593, -10.543767, 0.046148), ))
d11 = myAssembly.datums
myAssembly.PartitionCellByDatumPlane(datumPlane=d11[34], cells=pickedCells)

partInstances =(myAssembly.instances['SurfaceCasing-1'], )
myAssembly.seedPartInstance(regions=partInstances, size=0.013, deviationFactor=0.1, 
    minSizeFactor=0.1)
c1 = myAssembly.instances['SurfaceCasing-1'].cells
pickedRegions = c1.findAt(((0.233013, -1.8084, 0.030497), ))
myAssembly.generateMesh(regions=pickedRegions)

e1 = myAssembly.instances['SurfaceCasing-1'].edges
pickedEdges = e1.findAt(((-0.235, -11.75, 0.0), ), ((-0.2682, -5.25, 0.0), ), (
    (0.235, -11.75, 0.0), ), ((0.2682, -5.25, 0.0), ))
myAssembly.seedEdgeBySize(edges=pickedEdges, size=0.08, deviationFactor=0.1, 
    minSizeFactor=0.1, constraint=FINER)
c1 = myAssembly.instances['SurfaceCasing-1'].cells
pickedRegions = c1.findAt(((-0.233013, -10.666667, 0.030497), ))
myAssembly.generateMesh(regions=pickedRegions)

partInstances =(myAssembly.instances['Conductor-1'], )
myAssembly.seedPartInstance(regions=partInstances, size=0.015, deviationFactor=0.1, 
    minSizeFactor=0.1)
c1 = myAssembly.instances['Conductor-1'].cells
pickedRegions = c1.findAt(((0.352593, -1.7542, 0.046148), ))
myAssembly.generateMesh(regions=pickedRegions)

e1 = myAssembly.instances['Conductor-1'].edges
pickedEdges = e1.findAt(((-0.3556, -11.75, 0.0), ), ((-0.381, -5.25, 0.0), ), (
    (0.3556, -11.75, 0.0), ), ((0.381, -5.25, 0.0), ))
myAssembly.seedEdgeBySize(edges=pickedEdges, size=0.08, deviationFactor=0.1, 
    minSizeFactor=0.1, constraint=FINER)
c1 = myAssembly.instances['Conductor-1'].cells
pickedRegions = c1.findAt(((-0.352593, -10.666667, 0.046148), ))
myAssembly.generateMesh(regions=pickedRegions)

#-------------------Generating mesh----------------------------#
myPartInstance = (myAssembly.instances['BeamConductor-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=1.0, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

myPartInstance = (myAssembly.instances['BeamSurfaceCasing-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=1.0, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

myPartInstance = (myAssembly.instances['Cement-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=0.07, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

myPartInstance = (myAssembly.instances['ConductorHousing-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=0.015, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

myPartInstance = (myAssembly.instances['Wellhead-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=0.013, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

myPartInstance = (myAssembly.instances['BOP-1'], )
myAssembly.seedPartInstance(regions=myPartInstance, size=0.05, deviationFactor=0.1, minSizeFactor=0.1)
myAssembly.generateMesh(regions=myPartInstance)

elemType1 = mesh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD) #Linear: C3D8R #Quadratic: C3D20R 
elemType2 = mesh.ElemType(elemCode=C3D15, elemLibrary=STANDARD) #Linear: C3D6 #Quadratic: C3D15
elemType3 = mesh.ElemType(elemCode=C3D10, elemLibrary=STANDARD) #Linear: C3D4 #Quadratic: C3D10

c1 = myAssembly.instances['Cement-1'].cells
cells1 = c1.findAt(((-0.265932, -14.333333, 0.034806), ))
c2 = myAssembly.instances['Conductor-1'].cells
cells2 = c2.findAt(((-0.352593, -10.543767, 0.046148), ))
c3 = myAssembly.instances['ConductorHousing-1'].cells
cells3 = c3.findAt(((-0.446826, -1.056012, 0.031547), ))
c4 = myAssembly.instances['SurfaceCasing-1'].cells
cells4 = c4.findAt(((-0.233013, -10.570867, 0.030497), ))
c5 = myAssembly.instances['Wellhead-1'].cells
cells5 = c5.findAt(((-0.330289, -0.301535, 0.030911), ))
c6 = myAssembly.instances['Conductor-1'].cells
cells6 = c6.findAt(((0.352593, -1.7542, 0.046148), ))
c7 = myAssembly.instances['SurfaceCasing-1'].cells
cells7 = c7.findAt(((0.233013, -1.8084, 0.030497), ))
pickedRegions =((cells1+cells2+cells3+cells4+cells5+cells6+cells7), )
myAssembly.setElementType(regions=pickedRegions, elemTypes=(elemType1, elemType2, 
    elemType3))

#-------------Constraints between beam part of conductor and surface casing--------------#
#Same displacement in all degrees of freedom except vertical translation
nodes = myAssembly.instances['BeamConductor-1'].nodes
for i in xrange(len(nodes)):
    node = nodes[i:i+1]
    myAssembly.Set(nodes=node, name='BeamConductorNode-%d' % i)

nodes = myAssembly.instances['BeamSurfaceCasing-1'].nodes
for i in xrange(len(nodes)):
    node = nodes[i:i+1]
    myAssembly.Set(nodes=node, name='BeamSurfaceCasingNode-%d' % i)

    for j in xrange(1,7):
        if j == 3:
            continue

        myModel.Equation(name='Constraint-%d-DOF-%d' % (i,j), terms=((1.0, 'BeamSurfaceCasingNode-%d' % i, j), 
            (-1.0, 'BeamConductorNode-%d' % i, j)))


#-------------------------Boundary conditions-----------------------------#

#--------------------Fixed lower end of conductor-------------------------#
verts1 = myAssembly.instances['BeamConductor-1'].vertices.findAt(((0.0, -74.0, 0.0), ))
region = regionToolset.Region(vertices=verts1)
myModel.DisplacementBC(name='FixedConductor', createStepName='Initial', 
    region=region, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

#------------------Solid model symmetry conditions-----------------------#
f1 = myAssembly.instances['ConductorHousing-1'].faces
faces1 = f1.findAt(((-0.449049, -1.00317, 0.0), ), ((0.408111, -0.9129, 0.0), 
    ))
f2 = myAssembly.instances['Wellhead-1'].faces
faces2 = f2.findAt(((-0.329667, -0.30002, 0.0), ), ((0.255867, -0.156763, 0.0), 
    ))
f3 = myAssembly.instances['Conductor-1'].faces
faces3 = f3.findAt(((-0.372533, -6.087533, 0.0), ), ((0.364067, -6.087533, 
    0.0), ))
f4 = myAssembly.instances['SurfaceCasing-1'].faces
faces4 = f4.findAt(((-0.257133, -6.141733, 0.0), ), ((0.246067, -6.141733, 
    0.0), ))
f5 = myAssembly.instances['Cement-1'].faces
faces5 = f5.findAt(((-0.326467, -14.666667, 0.0), ), ((0.297333, -14.666667, 
    0.0), ))
region = myAssembly.Set(faces=faces1+faces2+faces3+faces4+faces5, name='SymmetryPlane')
myModel.DisplacementBC(name='Symmetry', createStepName='Initial', region=region, u1=UNSET, u2=UNSET, u3=SET, ur1=UNSET, ur2=UNSET, ur3=UNSET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

edges1 = myAssembly.instances['BOP-1'].edges.findAt(((0.0, 2.66275, 0.0), ))
region = regionToolset.Region(edges=edges1)
myModel.DisplacementBC(name='BOPSymmetry', createStepName='Initial', 
    region=region, u1=UNSET, u2=UNSET, u3=SET, ur1=SET, ur2=SET, ur3=UNSET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

#------Conductor/curface casing transition fixed against rotation-----------#
verts1 = myAssembly.instances['BeamConductor-1'].vertices.findAt(((0.0, -16.0, 0.0), ))
region = regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].DisplacementBC(name='Conductor', 
    createStepName='Initial', region=region, u1=UNSET, u2=UNSET, u3=UNSET, 
    ur1=SET, ur2=SET, ur3=SET, amplitude=UNSET, distributionType=UNIFORM, 
    fieldName='', localCsys=None)

verts1 = myAssembly.instances['BeamSurfaceCasing-1'].vertices.findAt(((0.0, -16.0, 0.0), ))
region = regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].DisplacementBC(name='Surface', createStepName='Initial', 
    region=region, u1=UNSET, u2=UNSET, u3=UNSET, ur1=SET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

#--------------------------------------Soil----------------------------------------------#
#Defining the nonlinear spring stiffness
f = open('PY.dat', 'r')

springValues = f.readlines()
springSoil = []

for line in springValues:
    value = line.split()
    springSoil.append(value)
    dummy = springValues.index(line)

    table = ((float(springSoil[springValues.index(line)][1]), float(springSoil[springValues.index(line)][0])), 
        (float(springSoil[springValues.index(line)][3]), float(springSoil[springValues.index(line)][2])),
        (float(springSoil[springValues.index(line)][5]), float(springSoil[springValues.index(line)][4])),
        (float(springSoil[springValues.index(line)][7]), float(springSoil[springValues.index(line)][6])),
        (float(springSoil[springValues.index(line)][9]), float(springSoil[springValues.index(line)][8])),
        (float(springSoil[springValues.index(line)][11]), float(springSoil[springValues.index(line)][10])),
        (float(springSoil[springValues.index(line)][13]), float(springSoil[springValues.index(line)][12])),
        (float(springSoil[springValues.index(line)][15]), float(springSoil[springValues.index(line)][14])),
        (float(springSoil[springValues.index(line)][17]), float(springSoil[springValues.index(line)][16])),
        (float(springSoil[springValues.index(line)][19]), float(springSoil[springValues.index(line)][18])),
        (float(springSoil[springValues.index(line)][21]), float(springSoil[springValues.index(line)][20])),
        (float(springSoil[springValues.index(line)][23]), float(springSoil[springValues.index(line)][22])),
        (float(springSoil[springValues.index(line)][25]), float(springSoil[springValues.index(line)][24])),
        (float(springSoil[springValues.index(line)][27]), float(springSoil[springValues.index(line)][26])),
        (float(springSoil[springValues.index(line)][29]), float(springSoil[springValues.index(line)][28])),
        (float(springSoil[springValues.index(line)][31]), float(springSoil[springValues.index(line)][30])),
        (float(springSoil[springValues.index(line)][33]), float(springSoil[springValues.index(line)][32])),) #(force [N], displacement[m])

    myModel.ConnectorSection(name='Soil-%d' % springValues.index(line), translationalType=AXIAL)
    elastic_0 = connectorBehavior.ConnectorElasticity(components=(1, ), behavior=NONLINEAR, table=table)
    elastic_0.ConnectorOptions()
    myModel.sections['Soil-%d' % springValues.index(line)].setValues(behaviorOptions =(elastic_0, ), extrapolation=LINEAR )

f.close

#-------Defining node sets for exterior conductor nodes and connecting them to reference points---------#
levels = [0.3, 0.8, 1.3, 1.8, 2.3, 2.8, 3.5, 5.01, 7.01, 9.01, 11.01] #Depth (m) below mudline for P-Y springs
delta = 0.04 #must be less than mesh size for conductor for one line of elements, less than two times mesh size for two lines of elements etc.

for level in levels:
    xmin = -1
    xmax = 1
    ymin = -1
    ymax = 1
    zmin = -4-level-delta
    zmax = -4-level+delta

    allNodes = myAssembly.instances['Conductor-1'].nodes
    levelNodes = allNodes.getByBoundingBox(xmin,zmin,ymin,xmax,zmax,ymax)

    myNodes = levelNodes[len(levelNodes)/2:len(levelNodes)]

    myAssembly.Set(nodes=myNodes, name='Level-%d' % levels.index(level))
    RP = myAssembly.ReferencePoint(point=(0.0, -4-level, 0.0))
    RP_id = RP.id
    refPoint = (myAssembly.referencePoints[RP_id], )

    region1=myAssembly.Set(referencePoints=refPoint, name='RP-%d' % levels.index(level))
    region2=myAssembly.sets['Level-%d' % levels.index(level)]
    myModel.MultipointConstraint(name='Soil-MPC-%d' % levels.index(level), 
    controlPoint=region1, surface=region2, mpcType=LINK_MPC, 
    userMode=DOF_MODE_MPC, userType=0, csys=None)

    #Connecting reference points to springs
    wire = myAssembly.WirePolyLine(points=((myAssembly.referencePoints[RP_id], None), ), mergeWire=False, meshable=False)
    oldName = wire.name
    myAssembly.features.changeKey(fromName=oldName, 
    toName='SoilSpring-%d' % levels.index(level))
    edges1 = myAssembly.edges.findAt(((0.0001, -4-level-0.0001, 0.0), ))
    myAssembly.Set(edges=edges1, name='SoilSpring-%d-Point' % levels.index(level))
    region = myAssembly.sets['SoilSpring-%d-Point' % levels.index(level)]
    csa = myAssembly.SectionAssignment(sectionName='Soil-%d' % levels.index(level), region=region)
    
levels = [13.0, 15.0, 17.0, 19.0, 24.0, 32.0, 40.0, 48.0, 56.0] #Depth (m) below mudline for P-Y springs
delta = 0.5 #must be less than mesh size for beam conductor for one line of elements, less than two times mesh size for two lines of elements etc.

for level in levels:

    RP = myAssembly.ReferencePoint(point=(0.0, -4-level, 0.0))
    RP_id = RP.id
    refPoint = (myAssembly.referencePoints[RP_id], )

    region1=regionToolset.Region(referencePoints=refPoint)
    region2=myAssembly.sets['BeamConductorNode-%d' % int(level-Transition)]
    myModel.Coupling(name='RefPointBeamConductor-%d' % levels.index(level), controlPoint=region1, 
        surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
        localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)

    #Connecting reference points to springs
    wire = myAssembly.WirePolyLine(points=((myAssembly.referencePoints[RP_id], None), ), mergeWire=False, meshable=False)
    oldName = wire.name
    myAssembly.features.changeKey(fromName=oldName, 
    toName='SoilSpring-%d' % (levels.index(level)+12))
    edges1 = myAssembly.edges.findAt(((0.0001, -4-level-0.0001, 0.0), ))
    myAssembly.Set(edges=edges1, name='SoilSpring-%d-Point' % (levels.index(level)+11))
    region = myAssembly.sets['SoilSpring-%d-Point' % (levels.index(level)+11)]
    csa = myAssembly.SectionAssignment(sectionName='Soil-%d' % (levels.index(level)+11), region=region)

#Template (lateral stiffness)
Templatestiffness = 
Gap = 
table = ((-Templatestiffness, -1.0), (0.0, -Gap), (0.0, 0.0), (0.0, Gap), (Templatestiffness, 1.0)) #(force [N], displacement[m]) 

myModel.ConnectorSection(name='Template', translationalType=AXIAL)
elastic_0 = connectorBehavior.ConnectorElasticity(components=(1, ), behavior=NONLINEAR, table=table)
elastic_0.ConnectorOptions()
myModel.sections['Template'].setValues(behaviorOptions =(elastic_0, ), extrapolation=LINEAR )

xmin = -1
xmax = 1
ymin = -1
ymax = 1
zmin = -3.7
zmax = -3.6

allNodes = myAssembly.instances['Conductor-1'].nodes
levelNodes = allNodes.getByBoundingBox(xmin,zmin,ymin,xmax,zmax,ymax)

myNodes = levelNodes[len(levelNodes)/2:len(levelNodes)]

myAssembly.Set(nodes=myNodes, name='TemplateNodes')
RP = myAssembly.ReferencePoint(point=(0.0, -3.66, 0.0))
RP_id = RP.id
refPoint = (myAssembly.referencePoints[RP_id], )

region1=myAssembly.Set(referencePoints=refPoint, name='RPTemplate')
region2=myAssembly.sets['TemplateNodes']
myModel.MultipointConstraint(name='Template-MPC', controlPoint=region1, surface=region2, mpcType=LINK_MPC, userMode=DOF_MODE_MPC, userType=0, csys=None)

wire = myAssembly.WirePolyLine(points=((myAssembly.referencePoints[RP_id], None), ), mergeWire=False, meshable=False)
oldName = wire.name
myAssembly.features.changeKey(fromName=oldName, toName='TemplateSpring')
edges1 = myAssembly.edges.findAt(((0.0001, -3.66-0.0001, 0.0), ))
myAssembly.Set(edges=edges1, name='TemplateSpring-Point')
region = myAssembly.sets['TemplateSpring-Point']
csa = myAssembly.SectionAssignment(sectionName='Template', region=region)

#----------------------------------Loads------------------------------------#

#-----------------BOP/LMRP weight - riser tension---------------------------#
BOPandtension = 
side1Edges1 = myAssembly.instances['BOP-1'].edges.findAt(((0.0, 0.0, 0.25), ))
region = regionToolset.Region(side1Edges=side1Edges1)
myModel.ShellEdgeLoad(name='BOP-LMRP-Weight-RiserTension', createStepName='Preload', 
    region=region, magnitude=-BOPandtension, distributionType=UNIFORM, field='',
    localCsys=None)

#----------------------Surface casing weight--------------------------------#
SCweight = 
verts1 = myAssembly.instances['BeamSurfaceCasing-1'].vertices.findAt(((0.0, -74.0, 0.0), ))
region = regionToolset.Region(vertices=verts1)
myModel.ConcentratedForce(name='SurfaceCasing-Weight', 
    createStepName='Preload', region=region, cf2=-SCweight, 
    distributionType=UNIFORM, field='', localCsys=None)

#-----------------------Riser response load--------------------------------#
Riserload = 
side1Edges1 = myAssembly.instances['BOP-1'].edges.findAt(((0.0, 10.651, 0.25), ))
region = regionToolset.Region(side1Edges=side1Edges1)
myModel.ShellEdgeLoad(name='RiserResponse', 
    createStepName='ResponseLoad', region=region, magnitude=Riserload,
    distributionType=UNIFORM, field='', localCsys=None, traction=TRANSVERSE)

#----------------------Request correct output------------------------------#
myModel.fieldOutputRequests['F-Output-1'].setValuesInStep(
    stepName='ResponseLoad', variables=('S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 
    'RF', 'CF', 'NFORC', 'NFORCSO', 'CSTRESS', 'CDISP'))
myModel.fieldOutputRequests['F-Output-1'].setValuesInStep(stepName='Preload', variables=(
    'S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 'RF', 'CF', 'NFORC', 'NFORCSO', 
    'CSTRESS', 'CDISP'))